//
//  ViewController.m
//  SynthesizerDemo
//  https://github.com/derkis/SynthesizerDemo
//
//  Created by Matthew Cobb on 4/1/12.
//  Copyright (c) 2012 UIC. All rights reserved.
//

#import "ViewController.h"

@interface ViewController ()

- (void)initGestures;
- (void)initSounds;

- (void)startBeat;
- (void)handleBeat:(NSTimer *)timer;

- (IBAction)handleSwipeDown:(UIGestureRecognizer *)sender;
- (IBAction)handleSwipeRight:(id)sender;
- (IBAction)handleSwipeLeft:(id)sender;

@end

@implementation ViewController

@synthesize snareSoundURL;
@synthesize snareSound;

@synthesize hardBeepSoundURL;
@synthesize hardBeepSound;

@synthesize echoBeepSoundURL;
@synthesize echoBeepSound;

@synthesize bursSoundURL;
@synthesize bursSound;

@synthesize loopLength;
@synthesize numberOfBeats;

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

#pragma mark - View lifecycle

// iOS methods automatically added to every controller

// this method is called once, when the view is created successfully
//      any custom setup for your view should be done in this method
- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self initGestures];
    [self initSounds];
    isPlaying = false;
    
    // giving the text fields default values
    loopLength.text = @"1";
    numberOfBeats.text = @"16";
}

// this method is called once, when the view is going out of scope
//      any necessary destruction should be done here
- (void)viewDidUnload {
    [self setLoopLength:nil];
    [self setNumberOfBeats:nil];
    [self setLoopLength:nil];
    [super viewDidUnload];
}

// this method is called whenever the user rotates the iDevice, to check if the rotation is legal
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return (interfaceOrientation == UIInterfaceOrientationLandscapeLeft ||
            interfaceOrientation == UIInterfaceOrientationLandscapeRight);
}

// GESTURE METHODS
// programatically create (some) gesture recognizers
- (void)initGestures {
    // create the gesutre recognizer
    UISwipeGestureRecognizer *swipeDownGestureRecognizer =     
        [[UISwipeGestureRecognizer alloc] initWithTarget:self 
                                                  action:@selector(handleSwipeDown:)];
    // specify the direction
    swipeDownGestureRecognizer.direction = UISwipeGestureRecognizerDirectionDown;
    // and bind it to the view
    [self.view addGestureRecognizer:swipeDownGestureRecognizer];
}

// swipe action handlers
// note the dots on the right
- (IBAction)handleSwipeDown:(UIGestureRecognizer *)sender {
    [beatsArray insertObject:snareSound atIndex:beatIndex];
}

- (IBAction)handleSwipeRight:(id)sender {
    [beatsArray insertObject:echoBeepSound atIndex:beatIndex];
}

- (IBAction)handleSwipeLeft:(id)sender {
    [beatsArray insertObject:hardBeepSound atIndex:beatIndex];
}

// a handler for the 'button pressed (aka touchUpInside)' action
- (IBAction)playPausePressed:(id)sender {
    isPlaying = !isPlaying;
    if(isPlaying) [self startBeat];
}

- (void)initSounds {
    NSURL *soundBundleURL;
    SystemSoundID soundID;
    
    soundBundleURL = [[NSBundle mainBundle] URLForResource: @"drum"
                                             withExtension: @"caf"];
    self.snareSoundURL = (__bridge_retained CFURLRef)soundBundleURL;
    AudioServicesCreateSystemSoundID(self.snareSoundURL, &soundID);
    self.snareSound = [NSNumber numberWithUnsignedLong:soundID];
    
    soundBundleURL = [[NSBundle mainBundle] URLForResource: @"hard_beep"
                                             withExtension: @"caf"];
    self.hardBeepSoundURL = (__bridge_retained CFURLRef)soundBundleURL;
    AudioServicesCreateSystemSoundID(self.hardBeepSoundURL, &soundID);
    self.hardBeepSound = [NSNumber numberWithUnsignedLong:soundID];
    
    soundBundleURL = [[NSBundle mainBundle] URLForResource: @"echo_beep"
                                             withExtension: @"caf"];
    self.echoBeepSoundURL = (__bridge_retained CFURLRef)soundBundleURL;
    AudioServicesCreateSystemSoundID(self.echoBeepSoundURL, &soundID);
    self.echoBeepSound = [NSNumber numberWithUnsignedLong:soundID];
    
    soundBundleURL = [[NSBundle mainBundle] URLForResource: @"burs"
                                             withExtension: @"caf"];
    self.bursSoundURL = (__bridge_retained CFURLRef)soundBundleURL;
    AudioServicesCreateSystemSoundID(self.bursSoundURL, &soundID);
    self.bursSound = [NSNumber numberWithUnsignedLong:soundID];
}

- (void)startBeat {
    beatIndex = 0;
    numBeats = [numberOfBeats.text integerValue];

    beatsArray = [[NSMutableArray alloc] initWithCapacity:numBeats];
    for(int i=0 ; i<numBeats ; i++) 
        [beatsArray insertObject:[NSNull null] atIndex:i];
    
    float interval = [self.loopLength.text floatValue] / numBeats;
    [NSTimer scheduledTimerWithTimeInterval:interval 
                                     target:self 
                                   selector:@selector(handleBeat:) 
                                   userInfo:@"beat"
                                    repeats:YES];
}

- (void)handleBeat:(NSTimer *)timer {
    if(!isPlaying) {
        [timer invalidate];
        return;
    }
    
    NSNumber *beat = [beatsArray objectAtIndex:beatIndex];    
    if(beat != (NSNumber *)[NSNull null])
        AudioServicesPlaySystemSound([beat unsignedLongValue]);
    beatIndex = ++beatIndex % numBeats;
}

@end
